

class PidController(object):

    def __init__(self, kp, ki, kd, state2err_fcn, a):
        """

        :param kp:
        :param ki:
        :param kd:
        :param state2err_fcn: (state, goal state) -> error
        :param a: k, | take 1d output to full u dimension
        """
        self.kp = kp
        self.ki = ki
        self.kd = kd
        self.state2err_fcn = state2err_fcn
        self.a = a
        self.x_goal = ()
        self.err_prev = 0.
        self.err_sum = 0.

    def set_setpoint(self, x_goal):
        self.x_goal = x_goal

    def reset_state(self):
        self.err_prev = 0.
        self.err_sum = 0.

    def __call__(self, x, t, **kwargs):
        err = self.state2err_fcn(x, self.x_goal)
        self.err_sum += err
        de_dt = err - self.err_prev
        u = self.kp * err + \
            self.ki * self.err_sum + \
            self.kp * de_dt
        self.err_prev = err
        return self.a * u




