import numpy as np


def make_control_signal(time_list, u_list):
    """

    :param time_list: [t1, t2, ..., tk] | times specifying intervals for controls
    - time assumed to start at 0, first interval being t \in [0, t1)
    - after T, u = 0 is supplied
    :param u_list: n, k | ith column is control corresponding to ith time interval
    :return:
        u_fcn: (x, t, **kwargs) -> n, | control input
    """
    n = len(time_list)
    assert u_list.shape[1] == n

    def u_fcn(x, t, **kwargs):
        if t < 0 or time_list[-1] <= t:
            return 0 * u_list[:, 0]
        i = np.digitize(t, time_list)
        return u_list[:, i]

    return u_fcn
