import numpy as np
import plotly.graph_objects as go
import scene_elements_3d.asteroid as ast
from vis import triangulated_surface as vtrs
from vis.xyz_utils import set_space_background


def main(is_surface_normals=False):
    seed = np.random.randint(1000)
    print('seed: {}'.format(seed))
    # seed = 0
    np.random.seed(seed)
    mu = np.array([
        [0., 0, 0],
        [2, 0, 1],
    ])
    sigma = np.array([
        [
            [2., 0, 0],
            [0, 1, 0],
            [0, 0, 1],
        ],
        [
            [1., 0, .2],
            [0, 1, 0],
            [.2, 0, 1],
        ],
    ])
    model = ast.generate_convex_asteroid(mu, sigma, 100)

    fig = go.Figure()
    fig.add_traces([vtrs.make_model_mesh(model)])
    if is_surface_normals:
        fig.add_traces(vtrs.make_surface_normals(
            model.vertices, model.triangles))
    title_str = 'Asteroid'
    fig.update_layout(
        title=title_str, autosize=False,
        width=1200, height=600,
        scene=dict(
            xaxis=dict(range=[-8, 8]),
            yaxis=dict(range=[-6, 6]),
            zaxis=dict(range=[-6, 6]),
        ), showlegend=False)
    set_space_background(fig)
    fig.show()


if __name__ == '__main__':
    main(is_surface_normals=False)
