import numpy as np
from rigid_body_models import boat
from state_space_models import empirical_drag
from controls import stepping
from controls import control_affine_clf
from vis import planar_thruster_model


def main_clf_rd1():
    DT = 0.1
    boat_model = boat.DiamondMountedThrusters()
    cts_sim_model = empirical_drag.EmpiricalDragCts(
        boat_model,
        np.array([.1, .5]) * 1.,
        np.array([.5, .1]) * 1.,
    )
    x0 = np.array([0., 0, 1, 3, np.pi/2, np.pi/4 * 1])
    x1 = np.array([0, 0, 0, 0, -np.pi/2, 0])
    m = x0.size
    selection_inds = [2, 3, 5]
    C = control_affine_clf.make_C_matrix(selection_inds, m)
    eps_rate = 3.
    min_norm_controller = control_affine_clf.MinNormRd1ClfController(eps_rate)
    min_norm_controller.set_setpoint(C, x1[selection_inds])
    T = 30
    x, u = stepping.solve_cts_eqns(
        cts_sim_model, x0, min_norm_controller, DT, T,
        cts_sim_model=cts_sim_model,
    )
    np.set_printoptions(suppress=True)
    print(x.T.round(2))
    print(u.T.round(2))
    planar_thruster_model.loop_sim(
        boat_model, x.T[:, [0, 1]], x.T[:, 4], u.T, DT)


def main_clf_rd2():
    DT = 0.1
    boat_model = boat.DiamondMountedThrusters()
    cts_sim_model = empirical_drag.EmpiricalDragCts(
        boat_model,
        np.array([.1, .5]) * 1.,
        np.array([.5, .1]) * 1.,
    )
    x0 = np.array([0., 0, 1, 2, np.pi/2, np.pi/4 * 1])
    x1 = np.array([0, 0, 0, 0, -np.pi/2, 0])

    eps_rate = 1.
    c_delta = 1.
    n = boat_model.f.shape[1]
    u_min = np.ones((n,)) * -2.
    u_max = np.ones((n,)) * 2.
    min_norm_controller = control_affine_clf.MinNormRd2ClfController(
        eps_rate, c_delta, u_min, u_max,
    )
    m = x0.size
    pos_selection_inds = [0, 1, 4]
    vel_selection_inds = [2, 3, 5]
    C = control_affine_clf.make_C_matrix(pos_selection_inds, m)
    C_dot = control_affine_clf.make_C_matrix(vel_selection_inds, m)
    min_norm_controller.set_setpoint(C, x1[pos_selection_inds], C_dot)

    T = 10
    x, u = stepping.solve_cts_eqns(
        cts_sim_model, x0, min_norm_controller, DT, T,
        cts_sim_model=cts_sim_model,
    )
    np.set_printoptions(suppress=True)
    print(x.T.round(2))
    print(u.T.round(2))
    planar_thruster_model.loop_sim(
        boat_model, x.T[:, [0, 1]], x.T[:, 4], u.T, DT)


if __name__ == '__main__':
    # main_clf_rd1()
    main_clf_rd2()
