import numpy as np
from rigid_body_models.shuttle import Shuttle
from state_space_models import quaternion_floating_integrator as qfli
from state_space_models import floating_integrator as fli
from misc import quaternion as qua
from controls import time_based
from controls import stepping
from vis import animate_3d as ani


def main_quaternion_integrator():
    DT = 0.1
    model = Shuttle(rho=1e1)
    cts_state_model = qfli.FreeFloating(model)
    x0 = np.array([
        0., 0, 0,
        0, 0, 0., 0,
        0, 0, 0,
        0, 0, 0,
    ])
    x0[3:7] = qua.exp_map2quaternion(np.array([0, 0, 1.0*np.pi/2]))
    print(x0[3:7], np.linalg.norm(x0[3:7]))
    u_fcn = time_based.make_control_signal(
        [1, 8],
        np.array([
            [0., 500., 0, 500],
            [0., 0, 0, 0],
        ]).T
    )
    T = 5
    x, u = stepping.solve_cts_eqns(cts_state_model, x0, u_fcn, DT, T)
    np.set_printoptions(suppress=True)
    print(x.T[-20:].round(2))

    scale = .01
    ani.loop_sim(model, x.T[:, :3], x.T[:, 3:7], u.T * scale, DT, is_quaternion=True)


def main_exp_map_integrator():
    DT = 0.1
    model = Shuttle(rho=1e1)
    cts_state_model = fli.FreeFloating(model)
    x0 = np.array([
        0., 0, 0,
        0, 0, 1.0 * np.pi / 2,
        0, 0, 0,
        0, 0, 0,
    ])
    u_fcn = time_based.make_control_signal(
        [1, 8],
        np.array([
            [0., 0., 1000, 1000],
            [0., 0, 0, 0],
        ]).T
    )
    T = 5
    x, u = stepping.solve_cts_eqns(cts_state_model, x0, u_fcn, DT, T)
    np.set_printoptions(suppress=True)
    print(x.T[-20:].round(2))

    scale = .01
    ani.loop_sim(model, x.T[:, :3], x.T[:, 3:6], u.T * scale, DT, is_quaternion=False)


if __name__ == '__main__':
    main_quaternion_integrator()
    # main_exp_map_integrator()
