import numpy as np
from rigid_body_models import boat
from state_space_models import simple_planar_integrator as ssm
from controls import stepping
from examples import rotatemove_pid
from vis import planar_thruster_model


def main():
    DT = 0.1
    boat_model = boat.DiamondMountedThrusters()
    cts_state_model = ssm.NoDragCts(boat_model)
    cts_state_model = ssm.AccelApproxDragCts(cts_state_model, 0.2)
    discrete_state_model = ssm.ZeroOrderHoldDiscrete(cts_state_model, DT)
    x0 = np.array([0., 0, 0, 0, np.pi/2, 0])

    pid_wp1 = rotatemove_pid.build_boat_rotate_move_controller()
    pid_wp1.set_setpoint(np.array([2, 2.]))
    T = 30
    x, u = stepping.solve_discrete_difference_eqns(
        discrete_state_model, x0, pid_wp1, DT, T)
    np.set_printoptions(suppress=True)
    print(x.T.round(2))
    planar_thruster_model.loop_sim(
        boat_model, x.T[:, [0, 1]], x.T[:, 4], u.T, DT)


if __name__ == '__main__':
    main()
