import numpy as np
from planners import prm
from scene_elements.scene import Scene
from scene_elements.sphere import Sphere
from scene_elements.water import Water
from scene_elements.markers import Markers
from time import time


def main():
    seed = np.random.randint(1000)
    # seed = 82
    np.random.seed(seed)
    print('seed: {}'.format(seed))

    n_obstacles = 20
    obstacle_xy = np.random.rand(n_obstacles, 2) * np.array([10, 8]) + \
        np.array([-5., 2])
    x_0 = np.array([0, 0.])
    x_goal = np.array([0, 12])
    t_0 = time()
    path_vertices = prm.prm(
        x_0, x_goal,
        obstacle_xy, np.ones((n_obstacles,)),
        sample_kwargs=dict(n_samples=8, mu_r=1.6, sig_r=.2),
    )
    print('planning time elapsed: {:.3f}s'.format(time() - t_0))

    colors = ['r', 'y', 'blue', 'black', 'green']
    scene = Scene(
        [Sphere(obstacle_xy[i], color=np.random.choice(colors))
         for i in range(obstacle_xy.shape[0])]
        + [Water()]
        + [Markers(path_vertices)]
    )

    import matplotlib.pyplot as plt

    fig, ax = plt.subplots()
    ax.set_xticks(np.mgrid[-50:50:1])
    ax.set_yticks(np.mgrid[-50:50:1])
    ax.set_xlim([-7, 7])
    ax.set_ylim([-1, 13])
    ax.set_aspect('equal')
    plt.grid(True)
    ax.set_axisbelow(True)
    scene.draw_top_view(ax)
    plt.show()


if __name__ == '__main__':
    main()
