import numpy as np
from scene_elements.scene import Scene
from scene_elements.sphere import Sphere
from scene_elements.water import Water
from scene_elements.camera import Camera
from misc.matrix_building import rot_yaw_xyz1
import matplotlib.pyplot as plt


def main():
    scene = Scene([
        Sphere([1, 2], color='green'),
        Sphere([-1, 2], color='red'),
        Sphere([2, 6], color='yellow'),
        Sphere([0, 7], color='black'),
        Water(),
    ])

    cam_f = 420.  # f * px/m scaling based on sensor size
    cam_rows = 480
    cam_cols = 640
    cam_K = np.array([
        [cam_f, 0, cam_rows/2, 0],
        [0, cam_f, cam_cols/2, 0],
        [0, 0, 1., 0],
    ])
    cam_Rt = np.array([
        [0, 0, 1, -.5],
        [0, -1, 0, 0],
        [1, 0, 0, 0],
        [0, 0, 0, 1],
    ])
    camera = Camera(K=cam_K, Rt=cam_Rt, cam_cols=cam_cols, cam_rows=cam_rows)
    body2world_Rt = rot_yaw_xyz1(np.pi/(2 + .2))
    camera.set_body2world_Rt(body2world_Rt)

    # make fig for top view and image view
    fig_scene, ax_scene = plt.subplots()
    scene.draw_top_view(ax_scene)
    ax_scene.set_xlim([-6, 6])
    ax_scene.set_ylim([-2, 10])
    ax_scene.set_aspect('equal')
    plt.show()

    fig_scene, ax_image = plt.subplots()
    scene.draw_image_view(ax_image, camera)
    ax_image.set_aspect('equal')
    plt.show()


if __name__ == '__main__':
    main()
