import numpy as np
from rigid_body_models import boat
from state_space_models import simple_planar_integrator as ssm
from state_space_models import empirical_drag
from controls import stepping
from controls import sdre
from vis import planar_thruster_model


def main_drag_mismatch():
    DT = 0.1
    boat_model = boat.DiamondMountedThrusters()
    cts_state_model = ssm.NoDragCts(boat_model)
    cts_state_model = ssm.AccelApproxDragCts(cts_state_model, 0.2)
    cts_sim_model = empirical_drag.EmpiricalDragCts(
        boat_model,
        np.array([5, 1]) * 2.,
        np.array([.5, 1]) * 1.,
    )
    x0 = np.array([0., 0, 0, 0, np.pi/2, 0])
    x1 = np.array([2, 2, 0, 0, -np.pi/2, 0])

    Q = np.diag([1., 1, 1, 1, 1, 2]) * 1.
    R = np.diag([1., 1, 1, 1]) * 1.
    sdre_controller = sdre.SdreController(Q, R)
    sdre_controller.set_setpoint(x1)
    T = 30
    x, u = stepping.solve_cts_eqns(
        cts_state_model, x0, sdre_controller, DT, T,
        cts_sim_model=cts_sim_model,
    )
    np.set_printoptions(suppress=True)
    print(x.T.round(2))
    print(u.T.round(2))
    planar_thruster_model.loop_sim(
        boat_model, x.T[:, [0, 1]], x.T[:, 4], u.T, DT)


def main():
    DT = 0.1
    boat_model = boat.DiamondMountedThrusters()
    cts_state_model = ssm.NoDragCts(boat_model)
    cts_state_model = ssm.AccelApproxDragCts(cts_state_model, 0.2)
    x0 = np.array([0., 0, 0, 0, np.pi/2, 0])
    x1 = np.array([2, 2, 0, 0, -np.pi/2, 0])

    Q = np.diag([1., 1, 1, 1, 1, 2]) * 1.
    R = np.diag([1., 1, 1, 1]) * 1.
    sdre_controller = sdre.SdreController(Q, R)
    sdre_controller.set_setpoint(x1)
    T = 30
    x, u = stepping.solve_cts_eqns(
        cts_state_model, x0, sdre_controller, DT, T)
    np.set_printoptions(suppress=True)
    print(x.T.round(2))
    planar_thruster_model.loop_sim(
        boat_model, x.T[:, [0, 1]], x.T[:, 4], u.T, DT)


if __name__ == '__main__':
    # main()
    main_drag_mismatch()
