import plotly.graph_objects as go
from rigid_body_models.shuttle import Shuttle
from vis import triangulated_surface as vtrs
from vis import triangulated_thruster_model as vttm
from vis.xyz_utils import set_space_background


def main_display_mesh(is_thrusters=True, is_space_bg=True):
    model = Shuttle(rho=1e6)
    fig = go.Figure()
    fig.add_traces([vtrs.make_model_mesh(model)])
    if is_thrusters:
        fig.add_traces(vttm.make_thrust_vectors(model))
        title_str = 'Shuttle thrusters'
    else:
        fig.add_traces(vtrs.make_surface_normals(
            model.vertices, model.triangles))
        title_str = 'Shuttle surface normals'
    fig.update_layout(
        title=title_str, autosize=False,
        width=1200, height=600,
        scene=dict(
            xaxis=dict(range=[-8, 8]),
            yaxis=dict(range=[-6, 6]),
            zaxis=dict(range=[-6, 6]),
        ), showlegend=is_thrusters)
    if is_space_bg:
        set_space_background(fig)
    fig.show()


if __name__ == '__main__':
    main_display_mesh(is_thrusters=True, is_space_bg=True)
