import numpy as np


def rot_2d(theta):
    """
    :param theta: rad
    :return: 2,2 | ccw rotation matrix
    """
    a = np.cos(theta)
    b = np.sin(theta)
    mat = np.array([[a, -b], [b, a]])
    return mat


def rot_yaw_xyz1(theta):
    """
    :param theta: rad
    :return: 4, 4 | homogeneous ccw rotation matrix
    """
    mat = np.eye(4)
    mat[:2, :2] = rot_2d(theta)
    return mat


def Rt_planar_xyz1(theta, t):
    """
    :param theta: yaw rad
    :param t: 2, | planar translation
    :return: 4, 4 | homogeneous ccw rotation matrix with translation
    """
    mat = rot_yaw_xyz1(theta)
    mat[:2, -1] = t
    return mat


def Rt_3d(R=(), t=()):
    """
    Make homogeneous transformation matrix
    - identity operation if not specified
    :param R: 3, 3
    :param t: 3,
    :return: 4, 4
    """
    if len(R) == 0:
        R = np.eye(3)
    if len(t) == 0:
        t = np.zeros((3,))
    Rt = np.eye(4)
    Rt[:3, :3] = R
    Rt[:3, -1] = t
    return Rt


