import numpy as np


class MountedThrusters(object):
    """
    For i = 1,...,k the ith mounted thruster has:
    r_i: 2, | offset from center of mass at 0,0 [m]
    f_i: 2, | unit vector in direction of force generated

    r: 2, k | matrix of offsets
    f: 2, k | matrix of directions

    m: total mass [kg]
    im: scalar for moment of inertia
    """
    r = ()
    f = ()
    m = 0.
    im = 0.


class DiamondMountedThrusters(object):
    """
    Body coordinate frame:
    ^ y
    |
    --> x (z-up, so right-handed)

      3       0
     ---------
    |  /   \  |
    |  \   /  |    with length in x and width in y
     ---------
      2       1
    """

    r = np.array([
        [1/4, 1/4, -1/4, -1/4],
        [1/4, -1/4, -1/4, 1/4],
    ])
    f = np.array([
        [1, 1, 1, 1],
        [-1, 1, -1, 1],
    ])/np.sqrt(2)
    m = 1.
    im = 1.5
    length = 1.
    width = 0.6



