import numpy as np


class Scene(object):

    def __init__(self, element_list):
        self.element_list = element_list

    def draw_top_view(self, ax):
        ax.grid(True)
        for element in self.element_list:
            element.draw_top_view(ax)

    def draw_image_view(self, ax, camera):
        """
        Draw all elements in the scene
        :param ax:
        :param camera:
        :return:
        """
        ax.set_xlim([0, camera.cam_cols])
        ax.set_ylim([0, camera.cam_rows])

        # apply world to cam body Rt
        cam_center_world = camera.get_camera_center_world()[:2]
        dist_from_cam = [np.linalg.norm(element.xy - cam_center_world)
                         if len(element.xy) > 0 else np.inf
                         for element in self.element_list]
        zorder_list = len(self.element_list) - np.argsort(np.asarray(dist_from_cam))
        for i in range(len(self.element_list)):
            self.element_list[i].draw_image_view(
                ax, camera, zorder=zorder_list[i])
