import numpy as np
import matplotlib.patches as pa


FAR_DIST_M = 1e5


class Water(object):

    def __init__(self, z=0., bound_xy=()):
        self.xy = ()
        self.z = z
        self.color = '#80ebff'
        self.bound_xy = bound_xy
        if len(bound_xy) == 0:
            theta = np.linspace(0, 2*np.pi, num=720, endpoint=False)
            self.bound_xy = np.array([
                np.cos(theta), np.sin(theta)
            ]) * FAR_DIST_M

    def draw_top_view(self, ax):
        m = 200
        rect = pa.Rectangle((-m/2, -m/2), width=m, height=m, facecolor=self.color, zorder=-1)
        ax.add_artist(rect)

    def draw_image_view(self, ax, camera, zorder=-1):
        """
        Draw "far out" water points that make the horizon line.
        Avoid using points that are nearly perpendicular to the camera's
        depth axis since these points will appear too close (small depth).
        Do not draw if behind camera.
        (Out of frame objects can be drawn, they will just be cut off)
        :param ax:
        :param camera:
        :param zorder: ordering for fake depth
        :return:
        """
        bound_xyz1 = np.vstack([self.bound_xy, 0*self.bound_xy])
        bound_xyz1[2, :] = self.z
        bound_xyz1[3, :] = 1
        bound_xy1 = camera.project(bound_xyz1)
        bound_xy1 = bound_xy1[:, bound_xy1[-1] > FAR_DIST_M/10]
        if bound_xy1.size == 0:
            return
        bound_xy1 /= bound_xy1[-1]
        # use furthest point on each side of camera fov
        # lower two points given by assumption we are surrounded by water
        ind_min = bound_xy1[1].argmin()
        ind_max = bound_xy1[1].argmax()
        verts = np.asarray([
            [0, 0],
            bound_xy1[:2, ind_min],
            bound_xy1[:2, ind_max],
            [0, camera.cam_cols],
        ])
        poly = pa.Polygon(verts[:, ::-1], facecolor=self.color, zorder=zorder)
        ax.add_artist(poly)
