import numpy as np
import plotly.graph_objects as go
from vis.xyz_utils import make_3d_vector
from misc.matrix_building import Rt_3d


def make_model_mesh(model, Rt=()):
    """
    :param model:
    :param Rt: 4, 4 | homogeneous coordinate rigid body transform
        - body frame to given frame
    :return:
    """
    if len(Rt) == 0:
        Rt = Rt_3d()
    xyz = Rt[:3, :3].dot(model.vertices.T).T + Rt[:3, -1]
    ijk = model.triangles
    try:
        color = model.color
    except AttributeError:
        color = None
    mesh = go.Mesh3d(
        x=xyz[:, 0], y=xyz[:, 1], z=xyz[:, 2],
        i=ijk[:, 0], j=ijk[:, 1], k=ijk[:, 2],
        color=color, flatshading=True
    )
    return mesh


def make_surface_normals(vertices, triangles):
    """
    :param vertices: n, 3
    :param triangles: m, 3 | m triangles specified by vertex indices
    :return:
    """
    object_list = []
    for ijk in triangles:
        u = vertices[ijk[1]] - vertices[ijk[0]]
        v = vertices[ijk[2]] - vertices[ijk[0]]
        n = np.cross(u, v)
        n /= np.linalg.norm(n)
        center = vertices[ijk].mean(axis=0)
        arrow_objs = make_3d_vector(center, n)
        object_list.extend(arrow_objs)
    return object_list
