import numpy as np
import plotly.graph_objects as go


def make_3d_vector(root, vector, scale=1., name=''):
    """
    Make a 3d vector for plotting
    :param root: 3,
    :param vector: 3,
    :param scale: multiplier for vector line and head length
    :param name:
    :return:
    """
    head_xyz = root + scale * vector
    xyz = np.vstack((root, head_xyz))
    line_obj = go.Scatter3d(
        x=xyz[:, 0],
        y=xyz[:, 1],
        z=xyz[:, 2],
        mode='lines',
        line=dict(width=4),
        name=name,
    )
    n = scale * vector / np.linalg.norm(vector)
    arrow_head_obj = go.Cone(
        x=[head_xyz[0]], y=[head_xyz[1]], z=[head_xyz[2]],
        u=[n[0]], v=[n[1]], w=[n[2]],
        showscale=False,
        name=name,
    )
    return line_obj, arrow_head_obj


def set_space_background(fig):
    """
    :param fig: plotly figure
    :return:
    """
    fig.update_layout(paper_bgcolor='rgb(10,10,10)')
    fig.update_scenes(xaxis_visible=False, yaxis_visible=False, zaxis_visible=False)
